// ***************************************************************
// This file was created using the CreateProject.sh script.
// CreateProject.sh is part of Bayesian Analysis Toolkit (BAT).
// BAT can be downloaded from http://www.mppmu.mpg.de/bat
// ***************************************************************

#include "XSecCombiModel.h"

#include <BAT/BCMath.h>

// ---------------------------------------------------------
XSecCombiModel::XSecCombiModel() : BCModel()
				 , fEfficiencyMu(0.35)
				 , fLuminosity(10.)
				 , fNobsMu(38)
				 , fNobsE(-1)
{
   // default constructor
   DefineParameters();
};

// ---------------------------------------------------------
XSecCombiModel::XSecCombiModel(const char * name) : BCModel(name)
{
   // constructor
   DefineParameters();
};

// ---------------------------------------------------------
XSecCombiModel::~XSecCombiModel()
   // default destructor
{
};

// ---------------------------------------------------------
void XSecCombiModel::DefineParameters()
{
   // Add parameters to your model here.
   // You can then use them in the methods below by calling the
   // parameters.at(i) or parameters[i], where i is the index
   // of the parameter. The indices increase from 0 according to the
   // order of adding the parameters.

  AddParameter("xsec", 2.0, 210.0); // index 0

}

// ---------------------------------------------------------
double XSecCombiModel::LogLikelihood(const std::vector<double> &parameters)
{
   // This methods returns the logarithm of the conditional probability
   // p(data|parameters). This is where you have to define your model.

   double logprob = 0.;

   // get parameters and calculate nuisance term
   double xsec   = parameters[0]; 

    // calculate efficiency as sum of estimated efficiency plus nuisance
   // term
   double effMu = fEfficiencyMu;

    // calculate luminosity as sum of estimated luminosity plus nuisance
   // term
   double luminosity = fLuminosity; 

   // check if efficiencies and luminosity are positive
   if (effMu < 0 || luminosity < 0)
      return -1e99;

   // calculate expected number of muon and electron events
   double nexpectedMu = xsec * luminosity * effMu;

   // Poisson probability for observing a certain number of events
   // given some expectation (muon channel)
   if (fNobsMu >= 0)
     logprob += BCMath::LogGaus(fNobsMu, nexpectedMu, sqrt(fNobsMu));
     //    logprob += BCMath::LogGaus(fNobsMu, nexpectedMu, sqrt(nexpectedMu));
     //     logprob += BCMath::LogPoisson(fNobsMu, nexpectedMu);


//   double x = parameters.at(0);
//   double y = parameters.at(1);
//   double eps = 0.5;

   // Breit-Wigner distribution of x with nuisance parameter y
//   logprob += BCMath::LogBreitWignerNonRel(x + eps*y, 0.0, 1.0);


   return logprob;
}

// ---------------------------------------------------------
double XSecCombiModel::LogAPrioriProbability(const std::vector<double> &parameters)
{
   // This method returns the logarithm of the prior probability for the
   // parameters p(parameters).

   double logprob = 0.;

   // get parameter range
   double dxsec = GetParameter(0)->GetRangeWidth(); 
   
   // flat prior for cross-section
   logprob += log(1./dxsec);

//   double x = parameters.at(0);
//   double y = parameters.at(1);

//   double dx = GetParameter(0)->GetRangeWidth();

//   logprob += log(1./dx);                    // flat prior for x
//   logprob += BCMath::LogGaus(y, 0., 1.0);   // Gaussian prior for y

   return logprob;
}
// ---------------------------------------------------------

